/*
 * DmxJam : The real tool for live lightshow!
 * "$Id: $" 
 *
 *                     Copyright (c) 2005, Jean-Marc Lienher
 *                        All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 * 
 *       Neither the names of the copyright holders nor the names of their 
 *       contributors may be used to endorse or promote products derived from 
 *       this software without specific prior written permission.
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * References :
 *
 * Changes :
 *	
 *
 */


#include <time.h>
#include <windows.h>
#include "util.h"

int gettimeofday(struct timeval *tp, struct timezone *tzp)
{
	DWORD tx;
	static int init = 0;
	static struct timeval tv;
	static DWORD lt;
	DWORD dt;
	if (!init) {
		SYSTEMTIME st;
		time_t ti;
		struct tm tk;
 
		GetLocalTime (&st);
		lt = timeGetTime();
		tk.tm_sec = st.wSecond;
		tk.tm_min = st.wMinute;
		tk.tm_hour = st.wHour;
		tk.tm_mday = st.wDay;
		tk.tm_mon = st.wMonth - 1;
		tk.tm_year = st.wYear - 1900;
		tk.tm_isdst = -1;
		ti = mktime (&tk);
		tv.tv_sec = (long)ti;
		tv.tv_usec = 0;
		init = 1;
	}
	//timeBeginPeriod(1);
	tx = timeGetTime();
	//timeEndPeriod(1);
	
	dt = tx - lt;
	{
		DWORD t;
		t = dt / 1000;
		tp->tv_sec = tv.tv_sec + t;
		dt -= t * 1000;
	}
	tp->tv_usec = tv.tv_usec + dt * 1000;
	if (tp->tv_usec >= 1000000) {
		tp->tv_usec -= 1000000;
		tp->tv_sec += 1;	
	}	
	return 0;
}

int usleep(unsigned long t)
{
	struct timeval tm, tva;

	gettimeofday(&tm, NULL);
	tm.tv_usec += t;
	if (tm.tv_usec >= 1000000) {
		tm.tv_usec -= 1000000;
		tm.tv_sec++;
	}
	if (t / 1000 > 1) {
		Sleep(t / 1000);
	}
	do {
		Sleep(0);
		gettimeofday(&tva, NULL);
	} while (!(tm.tv_sec < tva.tv_sec || (tm.tv_sec == tva.tv_sec && tm.tv_usec <= tva.tv_usec)));

	return 0;
}

#if defined(WIN32)
void *create_thread(Thread* t, void *(*f) (void *), void* p)
{
	*t = (Thread)_beginthread(
                        (void( __cdecl * )( void * ))f, 0, p);
	return (void*) *t;
}

void get_current_thread(Thread* t)
{
	*t = GetCurrentThread();
}

int wait_for_thread(Thread *t, int milisec) 
{
	int ret;
	ret = WaitForSingleObjectEx(*t, milisec, FALSE);

	return ret == WAIT_FAILED ? -1 : 0;
}

void init_thread_lock(ThreadLock *l)
{
	InitializeCriticalSection(l);
}

int lock_thread_lock(ThreadLock *l)
{
	EnterCriticalSection(l);
	return 0;
}



void unlock_thread_lock(ThreadLock *l)
{
	LeaveCriticalSection(l);
}



#else

int create_thread(Fl_Thread* t, void *(*f) (void *), void* p)
{
        return pthread_create((pthread_t*)t, 0, f, p);
}
#endif

/*
 * End of "$Id: $".
 */
